@@ -45,6 +45,7 @@
 #include <tqcheckbox.h>
 #include <ktempdir.h>
 #include <kprocess.h>
+#include <knuminput.h>
 #include <tdesu/process.h>
 #include <libtdeldap.h>
 #include <tdefiledialog.h>
@@ -145,6 +146,8 @@ LDAPController::LDAPController(TQWidget *parent, const char *name, const TQStrin
 	connect(m_base->ldapExportKey, TQT_SIGNAL(clicked()), this, TQT_SLOT(btnldapExportKey()));
 	connect(m_base->ldapExportCert, TQT_SIGNAL(clicked()), this, TQT_SLOT(btnldapExportCert()));
 
+	connect(m_base->crlRegenerate, TQT_SIGNAL(clicked()), this, TQT_SLOT(btncrlRegenerate()));
+
 	connect(m_base->btnChangeLDAPRootPassword, TQT_SIGNAL(clicked()), this, TQT_SLOT(btnChangeLDAPRootPassword()));
 	connect(m_base->btnChangeRealmAdminPassword, TQT_SIGNAL(clicked()), this, TQT_SLOT(btnChangeRealmAdminPassword()));
 
@@ -159,6 +162,11 @@ LDAPController::LDAPController(TQWidget *parent, const char *name, const TQStrin
 	connect(m_base->multiMasterReplicationMappings, TQT_SIGNAL(selectionChanged()), this, TQT_SLOT(multiMasterReplicationHighlighted()));
 	connect(m_base->multiMasterReplicationMappings, TQT_SIGNAL(executed(TQListViewItem*)), this, TQT_SLOT(modifySelectedMultiMasterReplication()));
 
+	connect(m_base->advancedCaCertExpiry, TQT_SIGNAL(valueChanged(int)), this, TQT_SLOT(caCertExpiryChanged()));
+	connect(m_base->advancedCaCrlExpiry, TQT_SIGNAL(valueChanged(int)), this, TQT_SLOT(caCrlExpiryChanged()));
+	connect(m_base->advancedKerberosCertExpiry, TQT_SIGNAL(valueChanged(int)), this, TQT_SLOT(kerberosCertExpiryChanged()));
+	connect(m_base->advancedLdapCertExpiry, TQT_SIGNAL(valueChanged(int)), this, TQT_SLOT(ldapCertExpiryChanged()));
+
 	m_fqdn = LDAPManager::getMachineFQDN();
 	m_roleFullyConfigured = true;
 
@@ -394,6 +402,10 @@ void LDAPController::load() {
 
 	// Load cert config
 	m_systemconfig->setGroup("Certificates");
+	m_certconfig.caExpiryDays = m_systemconfig->readNumEntry("caExpiryDays", KERBEROS_PKI_PEMKEY_EXPIRY_DAYS);
+	m_certconfig.caCrlExpiryDays = m_systemconfig->readNumEntry("caCrlExpiryDays", KERBEROS_PKI_CRL_EXPIRY_DAYS);
+	m_certconfig.kerberosExpiryDays = m_systemconfig->readNumEntry("kerberosExpiryDays", KERBEROS_PKI_KRB_EXPIRY_DAYS);
+	m_certconfig.ldapExpiryDays = m_systemconfig->readNumEntry("ldapExpiryDays", KERBEROS_PKI_LDAP_EXPIRY_DAYS);
 	m_certconfig.countryName = m_systemconfig->readEntry("countryName");
 	m_certconfig.stateOrProvinceName = m_systemconfig->readEntry("stateOrProvinceName");
 	m_certconfig.localityName = m_systemconfig->readEntry("localityName");
@@ -477,6 +489,11 @@ void LDAPController::load() {
 			m_base->caSetMaster->setEnabled(true);
 		}
 
+		m_base->advancedCaCertExpiry->setValue(m_certconfig.caExpiryDays);
+		m_base->advancedCaCrlExpiry->setValue(m_certconfig.caCrlExpiryDays);
+		m_base->advancedKerberosCertExpiry->setValue(m_certconfig.kerberosExpiryDays);
+		m_base->advancedLdapCertExpiry->setValue(m_certconfig.ldapExpiryDays);
+
 		updateCertDisplay();
 		m_certRefreshTimer.start(60*1000);
 	}
@@ -509,6 +526,13 @@ void LDAPController::updateCertDisplay() {
 	TQString ldap_certfile = LDAP_CERT_FILE;
 	ldap_certfile.replace("@@@ADMINSERVER@@@", m_realmconfig[m_defaultRealm].name.lower());
 
+	TQString realmname = m_defaultRealm.upper();
+	LDAPCredentials* credentials = new LDAPCredentials;
+	credentials->username = "";
+	credentials->password = "";
+	credentials->realm = realmname;
+	LDAPManager* ldap_mgr = new LDAPManager(realmname, "ldapi://", credentials);
+
 	// Certificate Authority
 	if (TQFile::exists(KERBEROS_PKI_PEM_FILE)) {
 		certExpiry = LDAPManager::getCertificateExpiration(KERBEROS_PKI_PEM_FILE);
@@ -574,6 +598,31 @@ void LDAPController::updateCertDisplay() {
 		m_base->ldapExpiryString->setText("File not found");
 		m_base->ldapExpiryString->setPaletteForegroundColor(CERT_STATUS_COLOR_NOTFOUND);
 	}
+
+	// Certificate Revocation List
+	TQByteArray certificateContents;
+	if (ldap_mgr->getTDECertificate("publicRootCertificateRevocationList", &certificateContents, NULL) == 0) {
+		certExpiry = LDAPManager::getCertificateExpiration(certificateContents);
+		if (certExpiry >= now) {
+			m_base->crlExpiryString->setText("Expires " + certExpiry.toString());
+			if (certExpiry >= soon) {
+				m_base->crlExpiryString->setPaletteForegroundColor(CERT_STATUS_COLOR_ACTIVE);
+			}
+			else {
+				m_base->crlExpiryString->setPaletteForegroundColor(CERT_STATUS_COLOR_STALE);
+			}
+		}
+		else {
+			m_base->crlExpiryString->setText("Expired " + certExpiry.toString());
+			m_base->crlExpiryString->setPaletteForegroundColor(CERT_STATUS_COLOR_EXPIRED);
+		}
+	}
+	else {
+		m_base->crlExpiryString->setText("File not found");
+		m_base->crlExpiryString->setPaletteForegroundColor(CERT_STATUS_COLOR_NOTFOUND);
+	}
+
+	delete ldap_mgr;
 }
 
 void LDAPController::btncaSetMaster() {
@@ -594,7 +643,7 @@ void LDAPController::btncaSetMaster() {
 			return;
 		}
 
-		LDAPManager::generatePublicKerberosCACertificate(m_certconfig);
+		LDAPManager::generatePublicKerberosCACertificate(m_certconfig, m_realmconfig[m_defaultRealm]);
 	
 		// Upload the contents of KERBEROS_PKI_PEM_FILE to the LDAP server
 		if (uploadKerberosCAFileToLDAP(ldap_mgr, &errorstring) != 0) {
@@ -608,7 +657,7 @@ void LDAPController::btncaSetMaster() {
 }
 
 void LDAPController::btncaRegenerate() {
-	LDAPManager::generatePublicKerberosCACertificate(m_certconfig);
+	LDAPManager::generatePublicKerberosCACertificate(m_certconfig, m_realmconfig[m_defaultRealm]);
 
 	TQString realmname = m_defaultRealm.upper();
 	LDAPCredentials* credentials = new LDAPCredentials;
@@ -687,6 +736,7 @@ void LDAPController::btnldapRegenerate() {
 	slapd_gid = pwd->pw_gid;
 
 	LDAPManager::generatePublicLDAPCertificate(m_certconfig, m_realmconfig[m_defaultRealm], slapd_uid, slapd_gid);
+	controlLDAPServer(SC_RESTART);
 
 	load();
 }
@@ -715,6 +765,26 @@ void LDAPController::btnldapExportCert() {
 	}
 }
 
+void LDAPController::btncrlRegenerate() {
+	TQString errstr;
+
+	// Bind to realm
+	TQString realmname = m_defaultRealm.upper();
+	LDAPCredentials* credentials = new LDAPCredentials;
+	credentials->username = "";
+	credentials->password = "";
+	credentials->realm = realmname;
+	LDAPManager* ldap_mgr = new LDAPManager(realmname, "ldapi://", credentials);
+
+	if (ldap_mgr->generatePKICRL(m_certconfig.caCrlExpiryDays, m_realmconfig[m_defaultRealm], KERBEROS_PKI_CRL_FILE, KERBEROS_PKI_PEMKEY_FILE, KERBEROS_PKI_CRLDB_FILE, &errstr) != 0) {
+		KMessageBox::error(this, i18n("<qt><b>Unable to regenerate CRL</b><p>Details: %1</qt>").arg(errstr), i18n("Unable to Regenerate CRL"));
+	}
+
+	delete ldap_mgr;
+
+	load();
+}
+
 void LDAPController::slotCertCopyResult(TDEIO::Job* job) {
 	if (job->error()) {
 		job->showErrorDialog(this);
@@ -728,11 +798,12 @@ void LDAPController::btnChangeLDAPRootPassword() {
 
 	bool ret = false;
 
-	TQCString rootPassword;
+	TQString rootPassword;
 	int result = KPasswordDialog::getNewPassword(rootPassword, i18n("Please enter the new LDAP root password:"));
 	if (result == KPasswordDialog::Accepted) {
 		SHA1 sha;
-		sha.process(rootPassword, strlen(rootPassword));
+		TQCString rootPassword2 = rootPassword.utf8();  // utf8 length could be different from TQString length
+		sha.process(rootPassword2, rootPassword2.length());
 		TQString rootpw_hash = sha.base64Hash();
 	
 		TQString oldconfigfilename = "/etc/ldap/slapd.d/cn=config/" + TQString("olcDatabase={%1}hdb.ldif.bkp").arg(1);
@@ -781,7 +852,7 @@ void LDAPController::btnChangeLDAPRootPassword() {
 }
 
 void LDAPController::btnChangeRealmAdminPassword() {
-	TQCString adminPassword;
+	TQString adminPassword;
 	int result = KPasswordDialog::getNewPassword(adminPassword, i18n("Please enter the new realm administrator password:"));
 	if (result == KPasswordDialog::Accepted) {
 		TQString realmname = m_defaultRealm.upper();
@@ -799,7 +870,7 @@ void LDAPController::btnChangeRealmAdminPassword() {
 			ldap_mgr->setPasswordForUser(adminuserinfo, &errorstring);
 
 			m_systemconfig->setGroup("Replication");
-			m_systemconfig->writeEntry("Password", adminPassword.data());
+			m_systemconfig->writeEntry("Password", adminPassword);
 			m_systemconfig->setGroup(NULL);
 			m_systemconfig->sync();
 		}
@@ -924,6 +995,30 @@ void LDAPController::defaults() {
 	//
 }
 
+void LDAPController::caCertExpiryChanged() {
+	m_certconfig.caExpiryDays = m_base->advancedCaCertExpiry->value();
+
+	emit(changed());
+}
+
+void LDAPController::caCrlExpiryChanged() {
+	m_certconfig.caCrlExpiryDays = m_base->advancedCaCrlExpiry->value();
+
+	emit(changed());
+}
+
+void LDAPController::kerberosCertExpiryChanged() {
+	m_certconfig.kerberosExpiryDays = m_base->advancedKerberosCertExpiry->value();
+
+	emit(changed());
+}
+
+void LDAPController::ldapCertExpiryChanged() {
+	m_certconfig.ldapExpiryDays = m_base->advancedLdapCertExpiry->value();
+
+	emit(changed());
+}
+
 void LDAPController::save() {
 	TQString prevRole = m_systemconfig->readEntry("LDAPRole", "Workstation");
 
@@ -938,6 +1033,10 @@ void LDAPController::save() {
 
 	// Write cert config
 	m_systemconfig->setGroup("Certificates");
+	m_systemconfig->writeEntry("caExpiryDays", m_certconfig.caExpiryDays);
+	m_systemconfig->writeEntry("caCrlExpiryDays", m_certconfig.caCrlExpiryDays);
+	m_systemconfig->writeEntry("kerberosExpiryDays", m_certconfig.kerberosExpiryDays);
+	m_systemconfig->writeEntry("ldapExpiryDays", m_certconfig.ldapExpiryDays);
 	m_systemconfig->writeEntry("countryName", m_certconfig.countryName);
 	m_systemconfig->writeEntry("stateOrProvinceName", m_certconfig.stateOrProvinceName);
 	m_systemconfig->writeEntry("localityName", m_certconfig.localityName);
@@ -950,17 +1049,21 @@ void LDAPController::save() {
 
 	m_systemconfig->sync();
 
+	TQString errorstring;
+	TQString realmname = m_defaultRealm.upper();
+	LDAPCredentials* credentials = new LDAPCredentials;
+	credentials->username = "";
+	credentials->password = "";
+	credentials->realm = realmname;
+	LDAPManager* ldap_mgr = new LDAPManager(realmname, "ldapi://", credentials);
+
+	if (ldap_mgr->setLdapCertificateStoreAttribute("publicRootCRLIntervalDays", TQString("%1").arg(m_certconfig.caCrlExpiryDays), &errorstring) != 0) {
+		KMessageBox::error(this, i18n("<qt><b>Unable to update CRL interval entry in LDAP database</b><p>Details: %1</qt>").arg(errorstring), i18n("LDAP Update Failure"));
+	}
+
 	// If role was not changed, update any role-specific advanced settings
 	if (prevRole == m_systemconfig->readEntry("LDAPRole", "Workstation")) {
 		if (m_base->systemRole->currentItem() == ROLE_PRIMARY_REALM_CONTROLLER) {
-			TQString realmname = m_defaultRealm.upper();
-			LDAPCredentials* credentials = new LDAPCredentials;
-			credentials->username = "";
-			credentials->password = "";
-			credentials->realm = realmname;
-			LDAPManager* ldap_mgr = new LDAPManager(realmname, "ldapi://", credentials);
-			TQString errorstring;
-	
 			// Write multi-master replication settings
 			LDAPMasterReplicationInfo replicationSettings;
 			replicationSettings.enabled = m_base->advancedEnableMultiMasterReplication->isChecked();
@@ -977,7 +1080,7 @@ void LDAPController::save() {
 			// Use the local password for inter-master authentication
 			// All realm controllers in a realm must (obviously) use the same admin/config password!
 			m_systemconfig->setGroup("Replication");
-			replicationSettings.syncPassword = m_systemconfig->readEntry("Password").utf8();
+			replicationSettings.syncPassword = m_systemconfig->readEntry("Password");
 			m_systemconfig->setGroup(NULL);
 			// Use the TDE LDAP CA for replication TLS
 			replicationSettings.caCertificateFile = KERBEROS_PKI_PEM_FILE;
@@ -990,6 +1093,8 @@ void LDAPController::save() {
 		}
 	}
 
+	delete ldap_mgr;
+
 	load();
 }
 
@@ -1521,7 +1626,7 @@ int LDAPController::setKerberosPasswordForUser(LDAPCredentials user, TQString *e
 		}
 		else if ((prompt.endsWith(" Password:")) && (prompt.startsWith(TQString(user.username + "@")))) {
 			kadminProc.enableLocalEcho(false);
-			kadminProc.writeLine(user.password, true);
+			kadminProc.writeLine(user.password.utf8(), true);
 			do { // Discard our own input
 				prompt = LDAPManager::readFullLineFromPtyProcess(&kadminProc);
 				printf("(kadmin) '%s'\n\r", prompt.ascii());
@@ -1529,7 +1634,7 @@ int LDAPController::setKerberosPasswordForUser(LDAPCredentials user, TQString *e
 			prompt = prompt.stripWhiteSpace();
 			if ((prompt.endsWith(" Password:")) && (prompt.startsWith("Verify"))) {
 				kadminProc.enableLocalEcho(false);
-				kadminProc.writeLine(user.password, true);
+				kadminProc.writeLine(user.password.utf8(), true);
 				do { // Discard our own input
 					prompt = LDAPManager::readFullLineFromPtyProcess(&kadminProc);
 					printf("(kadmin) '%s'\n\r", prompt.ascii());
@@ -1574,7 +1679,7 @@ int LDAPController::createRealmCertificates(LDAPCertConfig certinfo, LDAPRealmCo
 	chmod(KERBEROS_PKI_PEMKEY_FILE, S_IRUSR|S_IWUSR);
 	chown_safe(KERBEROS_PKI_PEMKEY_FILE, 0, 0);
 
-	LDAPManager::generatePublicKerberosCACertificate(certinfo);
+	LDAPManager::generatePublicKerberosCACertificate(certinfo, m_realmconfig[m_defaultRealm]);
 
 	// KDC certificate
 	TQString kdc_certfile = KERBEROS_PKI_KDC_FILE;
@@ -1790,7 +1895,7 @@ int LDAPController::createNewLDAPRealm(TQWidget* dialogparent, LDAPRealmConfig r
 	replacePlaceholdersInFile(templateDir + "sasl/slapd.conf", SASL_CONTROL_FILE, realmconfig, adminUserName, adminGroupName, machineAdminGroupName, standardUserGroupName, adminPassword, rootUserName, rootPassword);
 
 	// OpenSSL
-	replacePlaceholdersInFile(templateDir + "openssl/pki_extensions", OPENSSL_EXTENSIONS_FILE, realmconfig, adminUserName, adminGroupName, machineAdminGroupName, standardUserGroupName, adminPassword, rootUserName, rootPassword);
+	LDAPManager::writeOpenSSLConfigurationFile(realmconfig);
 
 	// FIXME
 	// This assumes Debian!
